% Apply alpha shapes method to understand baculum shape complexity
% Example code for a single baculum
% Gardiner, Benson and Brassey 2018
% j.d.gardiner@liverpool.ac.uk

close all
clearvars
clc

tic

% raw image data files to process

raw_image_filename = 'Aligned_Mustela_itatsi-141x286x995-8bit-0.0310953um.labels.raw';

% scan resolution of file

resolution = 0.0310953;

% sets fill radius for hole filling algorithm (must be integer)
% larger values will fill more holes (may result in
% oversmoothing of raw data)

fill_radius = 6; % 6 seems to be a good compromise between oversmoothing and missing holes
% see inside raw_to_filled_point_cloud.m
% function for use

% creates hole filled point cloud

fprintf('\nGenerating point cloud from raw file\n')

point_cloud = raw_to_filled_point_cloud_example(raw_image_filename,resolution,fill_radius);


% save point cloud size and caluclates bone volume based on scan
% resolution

original_point_cloud_size_filled = length(point_cloud);
original_volume = original_point_cloud_size_filled.*resolution^3;


% downsample point cloud to 10,000 points for further processing

fprintf(['\nDownsampling point cloud to 10,000 points\n' ...
    'Note - in the actual study the point cloud is downsampled to 100,000 but this\n' ...
    'slows the subsequent processes a lot. So for the example code 10,000 is used\n'])

downsample_size = 1e4;

point_cloud_index = randsample(length(point_cloud(:,1)),downsample_size);
point_cloud = point_cloud(point_cloud_index,:);


% caluclate the mean distance from each point to it's nearest
% 100 neighbours. Used to scale different size baculum, by
% multiplying by a refinement coefficient to set the alpha
% radii

fprintf(['\nCalculating distance to nearest 100 neighbours for each point\n' ...
    'Note - can be very slow for larger point clouds\n'])

[mean_distance_nearest_neighbour] = ...
    nearest_neighbour(point_cloud,100);


% create a distribution of refinement coefficients and multiply
% by mean distance from centroid and point cloud size to create
% alpha radii to use in alpha shape code. Each specimen has
% unique set of radii, but all use the same refinements.

refinement_coefficients = exp(linspace(6.91,-2.3,200));
radii = refinement_coefficients.*mean_distance_nearest_neighbour;

% loop to run the alpha shapes code for each coefficient

fprintf('\nRunning alpha-shapes code on point cloud for 200 refinement coefficients\n')

for iii = 1:length(refinement_coefficients)
    
    
    
    if ~rem(iii,10)
        
        percent_complete = 100.*(iii./...
            length(refinement_coefficients));
        
        fprintf('Percent complete = %.0f\n',percent_complete)
        
    end
    
    % call alphavol function (written by Jonas Lundgren) to caluclate alpha shape and volume
    
    [V(iii),S{iii}] = alphavol(point_cloud,radii(iii));
    
    
end


% run an optimisation routine to find best refinement coefficient

start_refinement = 10; %starting value for optimisation

fprintf('\nFinding optimal refinement coefficient via fminsearch optimisation function\n')

fun = @(refinement_coefficient)run_alpha_vol_for_optimisation(refinement_coefficient,...
    point_cloud,mean_distance_nearest_neighbour,original_volume);

[best_refinement,fval,exitflag,output] = fminsearch(fun,start_refinement);


% Once best refinement is found run alphavol function once more to save
% data from best alpha-shape fit.

[V_best,S_best] = alphavol(point_cloud,best_refinement.*mean_distance_nearest_neighbour);

difference = abs(refinement_coefficients - best_refinement);
best_refinement_closest_location = find(difference == min(difference));



% plot results

% Produce an plot showing relationship between refinement coefficent and
% alpha-shape volume. Inlcude example fits from four seperate locations.

fprintf('\nPlotting results\n\n')

positions_to_plot = [180 best_refinement_closest_location 80 1];
titles_to_plot = {'i','ii','iii','iv'};
colour_to_plot = [216 61 34]/256;

figure('color','w')

for i = 1:length(positions_to_plot(1,:))
    
    subplot(2,length(positions_to_plot(1,:)),i)
    
    if i == 2
        
        h = trisurf(S_best.bnd,...
            point_cloud(:,3),point_cloud(:,1),...
            point_cloud(:,2));
        
    else
        
        h = trisurf(S{positions_to_plot(i)}.bnd,...
            point_cloud(:,3),point_cloud(:,1),...
            point_cloud(:,2));
        
    end
    
    set(h,'facecolor',colour_to_plot,'edgecolor','none')
    light
    h.FaceLighting = 'gouraud';
    h.AmbientStrength = 0.3;
    h.DiffuseStrength = 0.8;
    h.SpecularStrength = 0.9;
    h.SpecularExponent = 25;
    axis equal
    axis off
    view(-60,30)
    title(titles_to_plot{i});
    
    p = get(gca,'position');
    
    adj = 0.07;
    
    p(1) = p(1)-adj;
    p(2) = p(2)-adj;
    p(3) = p(3)+adj;
    p(4) = p(4)+adj;
    
    set(gca,'position',p,'fontsize',12)
    
end

subplot(2,1,2)

h = plot(refinement_coefficients(positions_to_plot),...
    100*(V(positions_to_plot)./original_volume),'o');
set(h,'MarkerFaceColor',colour_to_plot,'markeredgecolor',colour_to_plot)
hold on
h = plot(refinement_coefficients,100*(V./original_volume),'k-');
set(h,'linewidth',1)



set(gca,'xscale','log')
box off
xlabel('Refinement coefficient')
ylabel({'Alpha-shape volume','(% of voxel volume)'})


h = refline(0,100);
set(h,'color','k')

text(1,80,['optimal alpha = ' num2str(best_refinement)])

for i = 1:length(positions_to_plot(1,:))
    
    
    h =  text(refinement_coefficients(positions_to_plot(i)),...
        30+100.*V(positions_to_plot(i))./original_volume,...
        titles_to_plot{i});
    
    set(h,'horizontalalignment','center','fontsize',12)
    
    
end

set(gca,'fontsize',12)



% Produce heat map plot of example species to highlight which areas
% contribute to different levels of refinement coefficient

figure('color','w')

points_used = [];
points_to_use = [];
store_points_to_use = [];

% for iii = 1:best_refinement_closest_location
for iii = 1:160
    
    current = S{iii}.bnd;
    unique_current = unique(current);
    
    if isempty(points_used)
        
        points_used = unique_current;
        points_to_use = unique_current;
        store_points_to_use{iii} = points_to_use;
        
    else
        
        count = 1;
        points_to_use = [];
        
        for jjj = 1:length(unique_current)
            
            if sum(points_used == unique_current(jjj))
                
            else
                points_to_use(count) = unique_current(jjj);
                count = count + 1;
                
            end
        end
        
        store_points_to_use{iii} = points_to_use;
        
        points_used = [points_used;points_to_use'];
        
    end
    
    
end


map_to_use = 'jet';
colour_to_plot = non_linear_colormap(map_to_use,length(store_points_to_use),4);

h = trisurf(S{160}.bnd,...
    point_cloud(:,3),point_cloud(:,1),...
    point_cloud(:,2));
set(h,'facecolor',[1 1 1],'edgecolor','none')


axis equal
axis off
view(-30,70)

hold on

for  iii = 1:length(store_points_to_use)
    
    
    
    h = plot3(point_cloud(store_points_to_use{iii},3),...
        point_cloud(store_points_to_use{iii},1),...
        point_cloud(store_points_to_use{iii},2),'.');
    set(h,'color',colour_to_plot(iii,:),'markersize',10)
    
end


colormap(colour_to_plot)
h_colorbar = colorbar;
ticks = get(h_colorbar,'limits');
ticks = [ticks(1) ticks(end)./2 ticks(end)];
% ticklabels = {'large','medium','small'};
ticklabels = {round(refinement_coefficients(1),4,'significant'),...
    round(refinement_coefficients(round(151./2)),4, 'significant'),...
    round(refinement_coefficients(151),4, 'significant')};


set(h_colorbar,'ticks',ticks,'ticklabels',ticklabels)

title({'points coloured according to the coarsest' ; 'alpha-shape to which they contribute'})

toc

